@@ -1,21 +1,27 @@
 #coding:utf-8
 """
+:Created: Nov 18, 2010
+
+:Author: prefer
+
 Классы для работы первично отображаемого интерфейса MIS.
 Включают список модулей в меню "Пуск" и список модулей на "Рабочем столе"
-
-Created on Nov 18, 2010
-
-@author: prefer
 """
+
 import threading
-import copy
 import inspect
+import itertools
+import warnings
 from uuid import uuid4
 
 from django.conf import settings
 from django.utils.importlib import import_module
 from django.contrib.auth.models import User, AnonymousUser
-from django.utils.log import logger
+try:
+    from django.utils.log import logger
+except ImportError:
+    from django.utils.log import getLogger
+    logger = getLogger('django')
 
 try:
     from m3_users import GENERIC_USER, SUPER_ADMIN
@@ -58,178 +64,209 @@ class DesktopException(Exception):
     pass
 
 
+class DesktopElementCollection(object):
+    """
+    Колекция эл-тов Рабочего Стола, определяющая их видимость
+    """
+    def __init__(self, filt=lambda x: True):
+        self._filt = filt
+        self._items = []
+        self.sorting_key = DEFAULT_SORTING
+
+    def __iter__(self):
+        return iter(self._items)
+
+    @property
+    def subitems(self):
+        """
+        Возвращает итератор, обеспечивающий фильтрацию и сортировку элементов
+        """
+        def prepared(item):
+            """
+            Возвращает элемент, либо None, если элемент отфильтрован
+            или содержит пустой список под-элементов
+            """
+            if not item.has_subitems:
+                result = item if self._filt(item) else None
+            else:
+                # на всякий случай клонируем
+                item = item.clone()
+                item.subitems = list(sorted(
+                    itertools.ifilter(
+                        bool,
+                        itertools.imap(
+                            prepared,
+                            item.subitems)),
+                    key=self.sorting_key
+                ))
+                if not item.subitems:
+                    result = None
+                else:
+                    result = item
+            return result
+
+        # обертка списка элементов в некое подобие LaunchGroup
+        stub = prepared(type('stub', (object,), {
+            'has_subitems': True,
+            'clone': lambda self: self,
+            'subitems': self._items[:]
+        })())
+        return iter(stub.subitems if stub else [])
+
+    def append(self, elem):
+        """
+        Добавление эл-та в коллекцию
+        :param elem: элемент рабочего стола
+        :type elem: BaseDesktopElement
+        """
+        self._items.append(elem)
+
+
 class DesktopModel(object):
     """
     Класс, агрегирующий в себе список модулей
     (start_menu и toolbox) в меню пуск
     и список модулей на Рабочем Столе (desktop)
     """
-
-    def __init__(self):
-        """
-        @param start_menu: список основных модулей для меню Пуск
-        @param toolbox: список настроечных модулей для меню Пуск
-        @param desktop: список модулей на Рабочем столе
-        @param toptoolbar: список модулей на верхней панели
-        """
-        self.start_menu = []
-        self.toolbox = []
-        self.desktop = []
-        self.toptoolbar = []
+    def __init__(self, request):
+        def filter_by_permissions(elem):
+            """
+            Возвращает True, если у ползователя есть права на пак элемента.
+            Работет только с DesktopShortcut'ами - у них есть атрибут pack,
+            остальные же элементы - отображаются всегда
+            :param elem: элемент рабочего стола
+            :type elem: DesktopShortcut
+            :return: has_perm - наличие права доступа к элементу
+            :rtype: bool
+            """
+            pack = getattr(elem, 'pack', None)
+            if pack is None or isinstance(pack, ActionPack):
+                return True
+            else:
+                return pack.has_perm(request)
+        self.start_menu = DesktopElementCollection(filter_by_permissions)
+        self.toolbox = DesktopElementCollection(filter_by_permissions)
+        self.desktop = DesktopElementCollection(filter_by_permissions)
+        self.toptoolbar = DesktopElementCollection(filter_by_permissions)
 
 
 class BaseDesktopElement(object):
-    '''
+    """
     Базовый класс для объекта модулей и объекта подменю
-    '''
+    """
+    # атрибуты, копируемые в при "клонировании" экземпляра
+    # потомки должны добавлять новые атрибуты (конкатенацией)
+    clonable_attrs = ('name', 'icon', 'index', 'id')
+
+    has_subitems = False
+
+    def t_is_subitems(self):
+        warnings.warn(
+            "\"t_has_items\" is deprecated! "
+            "Use \".has_subitems\"!",
+            FutureWarning
+        )
+        return self.has_subitems
+
     def __init__(self, *args, **kwargs):
-        '''
-        @param name: Название модуля или подменю
-        @param icon: Класс CSS, отвечающий за отрисовку иконки
-        '''
+        """
+        :param name: Название модуля или подменю
+        :type name: str
+        :param icon: Класс CSS, отвечающий за отрисовку иконки
+        :type icon: str
+        """
         self.name = ''
         self.icon = ''
         self.index = 0
         self.id = str(uuid4())[0:8]
 
     def _init_component(self, *args, **kwargs):
-        '''Заполняет атрибуты экземпляра значениями из kwargs'''
-        for k, v in kwargs.items():
+        """Заполняет атрибуты экземпляра значениями из kwargs"""
+        for k, val in kwargs.items():
             assert k in self.__dict__, (
                 'Instance attribute "%s" should be defined in class "%s"!' %
                 (k, self.__class__.__name__)
             )
-            self.__setattr__(k, v)
+            self.__setattr__(k, val)
 
-    def render(self):
-        '''
-        Должен быть переопределен в классе-наследнике
-        '''
-        raise NotImplementedError()
+    def __eq__(self, other):
+        return isinstance(other, BaseDesktopElement) and self.id == other.id
 
-    def __cmp__(self, obj):
-        res = isinstance(obj, BaseDesktopElement) and (self.id == obj.id)
-        return not res
+    def __cmp__(self, other):
+        assert isinstance(other, BaseDesktopElement)
+        return cmp(self.index, other.index) or cmp(self.name, other.name)
+
+    def clone(self, *args, **kwargs):
+        """
+        Возвращает "клон" себя
+        """
+        clone = self.__class__(*args, **kwargs)
+        for k in self.clonable_attrs:
+            setattr(clone, k, getattr(self, k))
+        return clone
 
 
 class DesktopLaunchGroup(BaseDesktopElement):
-    '''
+    """
     Класс для работы подменю по нажатию на кнопку Пуск
-    '''
+    """
+    has_subitems = True
+
     def __init__(self, *args, **kwargs):
-        '''
-        subitem: Хранит список модулей и список подменю
-        '''
+        """
+        :param subitem: Хранит список модулей и список подменю
+        :type subitem: list
+        """
         super(DesktopLaunchGroup, self).__init__(*args, **kwargs)
-        self.subitems = []
         self.index = 10
         self.icon = 'default-launch-group'
+        self.subitems = []
         self._init_component(*args, **kwargs)
 
-    def t_is_subitems(self):
-        '''
-        Для удобства понимания что рендерить. Используется в шаблонах.
-        '''
-        return True
-
-    def render(self):
-        '''
-        Рендерит имеющийся объект. Вызывается из функции render.
-        '''
-        if self.subitems:
-            return '{%s}' % ','.join([
-                'text: "%s"' % self.name.replace('"', "&quot;"),
-                'iconCls: "%s"' % self.icon,
-                'handler: function(){return false;}',
-                'menu: %s' % self.render_items(),
-            ])
-        else:
-            return None
-
-    def __deepcopy__(self, memo):
-        clone = DesktopLaunchGroup()
-        clone.name = self.name
-        clone.icon = self.icon
-        clone.index = self.index
-        clone.id = self.id
-        for subitem in self.subitems:
-            clone.subitems.append(
-                copy.deepcopy(subitem)
-            )
+    def clone(self, *args, **kwargs):
+        """
+        Возвращает "клон" себя
+        """
+        clone = super(DesktopLaunchGroup, self).clone(*args, **kwargs)
+        clone.subitems = self.subitems[:]
         return clone
 
-    def render_items(self):
-        '''
-        Рендерит имеющийся список объектов. Вызывается из шаблона.
-        '''
-        res = []
-        for item in self.subitems:
-            rendered = item.render()
-            if rendered:
-                res.append(rendered)
-        return '{items: [%s]}' % ','.join(res)
-
-    def ___str__(self):
-        return u'Группа: "%s" at %s' % (self.name, id(self))
-
 
 class DesktopLauncher(BaseDesktopElement):
-    '''
+    """
     Класс для работы модулей.
     Модули могут находится в меню Пуск, на рабочем столе.
     Данные модули могут включать в себя класс
     DesktopLaunchGroup в атрибут subitems
-    '''
+    """
+    clonable_attrs = BaseDesktopElement.clonable_attrs + ('url',)
+
     def __init__(self, *args, **kwargs):
-        '''
-        @param url: url-адрес, запрос по которому возвратит форму
-        '''
+        """
+        :param url: url-адрес, запрос по которому возвратит форму
+        :type url: str
+        """
         super(DesktopLauncher, self).__init__(*args, **kwargs)
         self.url = ''
         self.icon = 'default-launcher'
         self.index = 100
         self.context = {}  # словарь контекста
         self._init_component(*args, **kwargs)
-        self._set_default_handler()
 
-    def _set_default_handler(self):
-        self.handler = (
-            'function(){return sendRequest("%s", '
-            'AppDesktop.getDesktop(), %s);}'
-        ) % (
-            self.url,
-            self.t_render_context()
-        )
-
-    def t_is_subitems(self):
-        '''
-        Для удобства понимания что рендерить. Используется в шаблонах.
-        '''
-        return False
-
-    def t_render_context(self):
+    @property
+    def rendered_context(self):
+        """
+        Контекст в виде JSON
+        """
         return M3JSONEncoder().encode(self.context)
 
-    def render(self):
-        '''
-        Рендерит текущий объект.
-        Вызывается из метода render_items класса DesktopLaunchGroup
-        '''
-        return '{%s}' % ','.join([
-            'text:"%s"' % self.name.replace('"', "&quot;"),
-            'iconCls:"%s"' % self.icon,
-            (
-                'handler: function(){return sendRequest("%s",'
-                ' AppDesktop.getDesktop(), %s);}'
-            ) % (
-                self.url,
-                self.t_render_context()
-            ),
-            'scope: this',
-        ])
-
-    def ___str__(self):
-        return u'Ярлык: "%s" at %s' % (self.name, id(self))
+    def clone(self, *args, **kwargs):
+        """
+        Возвращает "клон" себя
+        """
+        clone = super(DesktopLauncher, self).clone(*args, **kwargs)
+        clone.context = self.context.copy()
+        return clone
 
 
 class DesktopShortcut(DesktopLauncher):
@@ -240,33 +277,32 @@ class DesktopShortcut(DesktopLauncher):
     def __init__(self, pack, *args, **kwargs):
         """
         Конструктор
-        @param pack: Имя или класс пака
+        :param pack: Имя или класс пака
+        :type pack: str / Action
+        :raise: DesktopException
         """
         super(DesktopShortcut, self).__init__(*args, **kwargs)
+        self.pack = pack
         # Если это экшен, то получаем его адрес
         if isinstance(pack, Action):
             self.url = pack.get_absolute_url()
             if not getattr(pack, 'title', None):
                 pack.title = getattr(pack.parent, 'title', '???')
         else:
-            if inspect.isclass(pack) and issubclass(pack, Action):
-                self.url = pack.absolute_url()
-                if not getattr(pack, 'title', None):
-                    pack.title = pack.__name__
-            else:
-                if not isinstance(pack, ActionPack):
-                    # Пробуем найти как пак
-                    pack = ControllerCache.find_pack(pack)
-                    if not pack:
-                        raise DesktopException(
-                            'Pack %s not found in ControllerCache' % pack)
+            if isinstance(pack, basestring):
+                # Пробуем найти как пак
+                pack = ControllerCache.find_pack(pack)
+                if not pack:
+                    raise DesktopException(
+                        'Pack %s not found in ControllerCache' % pack)
 
-                self.url = pack.get_list_url()
-                # Если не задано имя ярлыка, то название берем из справочника
-                if not kwargs.get('name', None):
-                    self.name = pack.title
+            self.url = pack.get_list_url()
+            # Если не задано имя ярлыка, то название берем из справочника
+            if not kwargs.get('name', None):
+                self.name = pack.title
 
-        self._set_default_handler()
+    def clone(self):
+        return super(DesktopShortcut, self).clone(self.pack)
 
 
 class MenuSeparator(BaseDesktopElement):
@@ -277,18 +313,19 @@ class MenuSeparator(BaseDesktopElement):
         super(MenuSeparator, self).__init__(*args, **kwargs)
         self._init_component(*args, **kwargs)
 
-    def render(self):
-        return '"-"'
+    def clone(self):
+        return self
 
 
 # Ключ сортировки элементов меню по умолчанию:
-SORTING = lambda x: x.index
+# сначала - по индексу, затем по имени пункта меню
+DEFAULT_SORTING = lambda x: (x.index, x.name)
 
 
 class DesktopLoader(object):
-    '''
+    """
     Загрузчик значков и меню для веб-десктопа
-    '''
+    """
     _lock = threading.RLock()
     _cache = None
     # Флаг того, что кэш загружен.
@@ -297,14 +334,18 @@ class DesktopLoader(object):
     # Флаг будет повторять ошибку для облегчения ее обнаружения.
     _success = False
 
-    #Константы определяющие куда добавить элемент
-    DESKTOP = 0
-    START_MENU = 1
-    TOOLBOX = 2
-    TOPTOOLBAR = 3
+    # Константы определяющие куда добавить элемент
+    DESKTOP = 0  # рабочий стол
+    START_MENU = 1  # меню "пуск"
+    TOOLBOX = 2  # правая панель меню "пуск"
+    TOPTOOLBAR = 3  # верхняя панель рабочего стола
 
     @classmethod
     def _load_desktop_from_apps(cls):
+        """
+        Загрузка элементов Раб.Стола, декларированных в приложениях проекта
+        :raise: ImportError
+        """
         cls._lock.acquire()
         try:
             if not cls._success:
@@ -319,7 +360,7 @@ class DesktopLoader(object):
                                 err.args[0].find('app_meta') == -1):
                             logger.exception(
                                 u'При сборке интерфейса не удалось '
-                                u'подключить %s' % app_name
+                                u'подключить %s', app_name
                             )
                             raise
                         continue
@@ -332,16 +373,25 @@ class DesktopLoader(object):
 
     @classmethod
     def add_el_to_desktop(cls, desktop, metarole_code):
-        '''
+        """
         Добавляет элементы к интерфейсу в зависимости от метароли
-        '''
+        :param desktop: экземпляр DesktopModel
+        :type desktop: obj
+        :param metarole_code: код метароли
+        :type metarole_code: str
+        """
         def join_list(existing_list, in_list):
-            '''
+            """
             Складывает два списка с объектами
             DesktopLaunchGroup и DesktopLauncher
             произвольного уровня вложенности.
             Критерием равенства является имя элемента!
-            '''
+            :param existing_list: список элементов
+            :type existing_list: list
+            :param in_list: список элементов
+            :type in_list: list
+
+            """
             names_dict = {}
             for existing_el in existing_list:
                 assert isinstance(existing_el, BaseDesktopElement)
@@ -379,23 +429,23 @@ class DesktopLoader(object):
                 join_list(desktop.toptoolbar, items)
 
     @classmethod
-    def sort_desktop(cls, desktop_list, sorting=SORTING):
-        '''
+    def sort_desktop(cls, desktop_list, sorting=DEFAULT_SORTING):
+        """
         Сортирует все контейнеры десктопа в зависимости от индекса (index)
-        '''
-
-        desktop_list.sort(key=sorting)
-        for item in desktop_list:
-            if isinstance(item, DesktopLaunchGroup):
-                cls.sort_desktop(item.subitems)
+        """
+        desktop_list.sorting_key = sorting
 
     @classmethod
-    def populate(cls, user, desktop, sorting=SORTING):
-        '''
+    def populate(cls, user, desktop, sorting=DEFAULT_SORTING):
+        """
         Метод, который выполняет всю работу по настройке десктопа во вьюшке
-        @user - пользователь или None
-        @desktop - экземпляр DesktopModel
-        '''
+        :param user: пользователь или None
+        :type user: django.contrib.auth.models.User
+        :param desktop: экземпляр DesktopModel
+        :type desktop: DesktopModel
+        :param sorting: функция сортировки
+        :type sorting: callable
+        """
         assert isinstance(user, (User, AnonymousUser))
 
         roles = []
@@ -407,12 +457,16 @@ class DesktopLoader(object):
         cls.populate_desktop(desktop, roles, sorting)
 
     @classmethod
-    def populate_desktop(cls, desktop, roles=None, sorting=SORTING):
+    def populate_desktop(cls, desktop, roles=None, sorting=DEFAULT_SORTING):
         """
         Построение Раб.Стола по элементам @desktop(экземпляр DesktopModel),
         относительно набора ролей @roles.
-        Этот метод НЕ ТРЕБУЕТ django.contrib.auth
-        и экземпляра User в частности.
+        .. note ::
+            Метод НЕ ТРЕБУЕТ django.contrib.auth и экземпляра User в частности
+        :param desktop: экземпляр DesktopModel
+        :type desktop: DesktopModel
+        :param roles: список ролей
+        :type roles: list
         """
         assert isinstance(desktop, DesktopModel)
 
@@ -427,62 +481,77 @@ class DesktopLoader(object):
 
     @classmethod
     def add(cls, metarole, place, element):
-        '''
+        """
         Добавление элемента дектопа в кэш заданной метароли
-        '''
+        :param metarole: роль
+        :type metarole: str
+        :param place: место размещения элемента (рабочий стол, меню и тулбоксы)
+        :type place: int
+        :raise: AssertionError
+        """
         def find_by_name(existed_list, name):
-            '''
+            """
             Поиск внутри списка группы с заданным именем name
-            '''
+            :param existed_list: список группы
+            :type existing_list: list
+            :param name: имя
+            :type name: unicode
+            """
             for item in existed_list:
-                if isinstance(item, DesktopLaunchGroup):
-                    assert isinstance(item.name, unicode), (
-                        'The attribute "name" must be written in Unicode')
-                    if item.name == name:
-                        return item
+                if isinstance(item, DesktopLaunchGroup) and item.name == name:
+                    return item
 
         def insert_item(existed_list, item):
-            # Если добавляемый элемент группа,
-            # то нужно проверить есть ли у нас уже такая группа.
-            # Если нет - добавляем, иначе нужно зайти в нее
-            # и продолжить проверку вниз по дереву
-
+            """
+            Если добавляемый элемент - группа,
+            то нужно проверить есть ли у нас уже такая группа.
+            Если нет - добавляем, иначе нужно зайти в нее
+            и продолжить проверку вниз по дереву
+            :param existed_list: список группы
+            :type existing_list: list
+            :param item: элемент
+            :type item: DesktopShortcut / DesktopLauncher
+            """
             if isinstance(item, DesktopLaunchGroup):
-
                 collision_item = find_by_name(existed_list, item.name)
-
                 if collision_item is None:
                     # Раз нет переcений по имени, то можно добавлять
-
                     existed_list.append(item)
                 else:
-                    for it in item.subitems:
-                        insert_item(collision_item.subitems, it)
-            else:
-                if item not in existed_list:
-                    existed_list.append(item)
+                    for i in item.subitems:
+                        insert_item(collision_item.subitems, i)
+            elif item not in existed_list:
+                existed_list.append(item)
 
-        def insert_for_role(metarole, el, processed_metaroles=[]):
-            if metarole in processed_metaroles:
-                return
-            processed_metaroles.append(metarole)
-            element = copy.deepcopy(el)
+        def insert_for_role(metarole, elem, processed_metaroles):
+            """
+            Добавление элемента для метароли
+            :param metarole: роль
+            :type metarole: str
+            :param elem: элемент
+            :type elem: DesktopShortcut / DesktopLauncher
+            :param processed_metaroles: обработанные метароли
+            :type processed_metaroles: list
+            """
+            if metarole not in processed_metaroles:
+                processed_metaroles.append(metarole)
+                element = elem.clone()
 
-            # Кэш состоит из 3х уровней: словарь с метаролями,
-            # словарь с местами и список конечных элементов
-            items_for_role = cls._cache.get(metarole.code, {})
-            items_for_place = items_for_role.get(place, [])
-            insert_item(items_for_place, element)
-            items_for_role[place] = items_for_place
-            cls._cache[metarole.code] = items_for_role
+                # Кэш состоит из 3х уровней: словарь с метаролями,
+                # словарь с местами и список конечных элементов
+                items_for_role = cls._cache.get(metarole.code, {})
+                items_for_place = items_for_role.get(place, [])
+                insert_item(items_for_place, element)
+                items_for_role[place] = items_for_place
+                cls._cache[metarole.code] = items_for_role
 
-            # Недостаточно добавить элементы только в одну метароль,
-            # т.к. она может входить внутрь другой метароли.
-            # Так что нужно пробежаться по ролям,
-            # которые включают в себя нашу роль.
+                # Недостаточно добавить элементы только в одну метароль,
+                # т.к. она может входить внутрь другой метароли.
+                # Так что нужно пробежаться по ролям,
+                # которые включают в себя нашу роль.
 
-            for role in metarole.get_owner_metaroles():
-                insert_for_role(role, element, processed_metaroles)
+                for role in metarole.get_owner_metaroles():
+                    insert_for_role(role, element, processed_metaroles)
 
         #===============================================
         assert place in (
@@ -495,7 +564,7 @@ class DesktopLoader(object):
             metarole = get_metarole(metarole)
         assert isinstance(metarole, UserMetarole)
 
-        insert_for_role(metarole, element)
+        insert_for_role(metarole, element, processed_metaroles=[])
 
 
 #==============================================================================
@@ -503,21 +572,27 @@ class DesktopLoader(object):
 #==============================================================================
 
 def add_desktop_launcher(
-        name='', url='', icon='', path=None, metaroles=None, places=None):
-    '''
+    name='', url='', icon='', path=None, metaroles=None, places=None
+):
+    """
     Шорткат для добавления ланчеров в элементы рабочего стола.
 
-    @param name: подпись ланчера на рабочем столе/в меню
-    @param url: url, по которому происходит обращение к ланчеру
-    @param icon: класс, на основе которого рисуется иконка ланчера
-    @param path: список кортежей, которые задают путь к ланчеру в случае, если
-                 этот ланчер спрятан в подменю.
-                 Каждый элемент пути задается либо в одном из форматов:
-                     "(название,)",
-                     "(название, иконка,)",
-                     "(название, иконка, индекс)"
-    @param metaroles: список метаролей (либо одиночная метароль)
-    '''
+    :param name: подпись ланчера на рабочем столе/в меню
+    :type name: str
+    :param url: url, по которому происходит обращение к ланчеру
+    :type url: str
+    :param icon: класс, на основе которого рисуется иконка ланчера
+    :type icon: m3_ext.icons.Icon
+    :param path: список кортежей, которые задают путь к ланчеру в случае, если
+                этот ланчер спрятан в подменю.
+    .. note::
+                Каждый элемент пути задается либо в одном из форматов:
+                    "(название,)",
+                    "(название, иконка,)",
+                    "(название, иконка, индекс)"
+    :type path: list
+    :param metaroles: список метаролей (либо одиночная метароль)
+    """
     if not metaroles or not places:
         return
 
@@ -552,11 +627,8 @@ def add_desktop_launcher(
         launcher = root  # мы в ланчеры, значится, будем добавлять самого рута.
 
     for metarole in cleaned_metaroles:
-        mt = (
-            get_metarole(metarole)
-            if isinstance(metarole, basestring)
-            else metarole
-        )
-        if mt:
+        if isinstance(metarole, basestring):
+            metarole = get_metarole(metarole)
+        if metarole:
             for place in cleaned_places:
-                DesktopLoader.add(mt, place, launcher)
+                DesktopLoader.add(metarole, place, launcher)
