@@ -4,6 +4,7 @@ Created on 3.3.2010
 
 @author: prefer
 """
+import json
 from django.conf import settings
 from django.utils.datastructures import SortedDict
 
@@ -11,140 +12,6 @@ from m3_ext.ui.base import ExtUIComponent, BaseExtComponent
 from base import BaseExtPanel
 
 
-#==============================================================================
-class ExtPivotGrid(BaseExtPanel):
-    """
-    Сводная таблица
-    """
-    def __init__(self, *args, **kwargs):
-        super(ExtPivotGrid, self).__init__(*args, **kwargs)
-        self.template = 'ext-grids/ext-pivot-grid.js'
-        self.__store = None
-        self.aggregator = None
-        self.renderer = None
-        self.measure = None
-        self._left_axis = []
-        self._top_axis = []
-        self._items = []
-        self.__cm = None
-        self.col_model = ExtGridDefaultColumnModel()
-        self.force_fit = True
-        self.auto_fill = True
-        self.__view = None
-        self._view_config = {}
-        self.init_component(*args, **kwargs)
-
-    def t_render_store(self):
-        return self.__store.render(self.columns)
-
-    def t_render_columns(self):
-        return self.t_render_items()
-
-    def add_column(self, **kwargs):
-        self.columns.append(ExtGridColumn(**kwargs))
-
-    def add_left_axis(self, **kwargs):
-        self.left_axis.append(ExtPivotGridAxis(**kwargs))
-
-    def add_top_axis(self, **kwargs):
-        self.top_axis.append(ExtPivotGridAxis(**kwargs))
-
-    def render_base_config(self):
-        super(ExtPivotGrid, self).render_base_config()
-        self._view_config['forceFit'] = self.force_fit
-        self._view_config['autoFill'] = self.auto_fill
-        for args in (
-            ('store', self.t_render_store, self.get_store()),
-            ('measure', self.measure),
-            ('aggregator', self.aggregator),
-            ('renderer', self.renderer),
-            ('leftAxis', self.t_render_left_axis),
-            ('topAxis', self.t_render_top_axis),
-            ('colModel', self.col_model.render),
-            ('view', self.t_render_view, self.view),
-            ('viewConfig', self._view_config),
-        ):
-            self._put_config_value(*args)
-
-    @property
-    def columns(self):
-        return self._items
-
-    @property
-    def left_axis(self):
-        return self._left_axis
-
-    @property
-    def top_axis(self):
-        return self._top_axis
-
-    @property
-    def view(self):
-        return self.__view
-
-    @view.setter
-    def view(self, value):
-        self.__view = value
-
-    def render(self):
-        self.render_base_config()
-        self.render_params()
-        config = self._get_config_str()
-        params = self._get_params_str()
-        return 'new Ext.grid.PivotGrid({%s}, {%s})' % (config, params)
-
-    def set_store(self, store):
-        self.__store = store
-
-    def get_store(self):
-        return self.__store
-
-    store = property(get_store, set_store)
-
-    @property
-    def col_model(self):
-        return self.__cm
-
-    @col_model.setter
-    def col_model(self, value):
-        self.__cm = value
-        self.__cm.grid = self
-
-    def t_render_view(self):
-        return self.view.render()
-
-    def t_render_left_axis(self):
-        return '[%s]' % ','.join(
-            ['''{
-            dataIndex:"%s",
-            header:"%s",
-            width:%d,
-            defaultHeaderWidth:%d,
-            orientation:"%s"
-            }''' % (
-            axe.data_index,
-            axe.header,
-            axe.width,
-            axe.default_header_width,
-            axe.orientation) for axe in self.left_axis])
-
-    def t_render_top_axis(self):
-        return '[%s]' % ','.join(
-            ['''{
-            dataIndex:"%s",
-            header:"%s",
-            width:%d,
-            defaultHeaderWidth:%d,
-            orientation:"%s"
-            }''' % (
-            axe.data_index,
-            axe.header,
-            axe.width,
-            axe.default_header_width,
-            axe.orientation) for axe in self.top_axis])
-
-
-#==============================================================================
 class ExtGrid(BaseExtPanel):
     """
     Таблица (Grid)
@@ -221,15 +88,21 @@ class ExtGrid(BaseExtPanel):
         self.banded_columns = SortedDict()
 
     def t_render_plugins(self):
-        # FIXME: Вынести в lambda функцию
-        return '[%s]' % ','.join(self.plugins)
+        """
+        Рендеринг плагинов
+        """
+        res = []
+        for plugin in self.plugins:
+            res.append(plugin.render() if hasattr(plugin, 'render') else plugin)
+
+        return '[%s]' % ','.join(res)
 
     def t_render_banded_columns(self):
-        '''
+        """
         Возвращает JS массив состоящий из массивов с описанием объединенных
         колонок. Каждый вложенный массив соответствует уровню шапки грида от
         верхней к нижней.
-        '''
+        """
         result = []
         for level_list in self.banded_columns.values():
             result.append('[%s]' % ','.join(
@@ -247,49 +120,50 @@ class ExtGrid(BaseExtPanel):
         return self.__cm.render()
 
     def add_column(self, **kwargs):
-        '''
+        """
         Добавляет стандартную колонку
-        '''
+        """
         self.columns.append(ExtGridColumn(**kwargs))
 
     def add_bool_column(self, **kwargs):
-        '''
+        """
         Добавляет булевую колонку
-        '''
+        """
         self.columns.append(ExtGridBooleanColumn(**kwargs))
 
     def add_check_column(self, **kwargs):
-        '''
+        """
         Добавляет колонку для выбора значения
-        '''
+        """
         self.columns.append(ExtGridCheckColumn(**kwargs))
 
     def add_number_column(self, **kwargs):
-        '''
+        """
         Добавляет числовую колонку
-        '''
+        """
         self.columns.append(ExtGridNumberColumn(**kwargs))
 
     def add_date_column(self, **kwargs):
-        '''
+        """
         Добавляет колонку с датой
-        '''
+        """
         self.columns.append(ExtGridDateColumn(**kwargs))
 
     def add_banded_column(self, column, level, colspan):
-        '''
+        """
         Добавляет в грид объединенную ячейку.
-        @param column: Колонка грида (ExtGridColumn)
-        @param colspan: Количество колонок которые находятся
-            под данной колонкой (int)
-        @param level: Уровень учейки где
-          0 - самый верхний,
-          1-ниже, и т.д. (int)
+        :param column: Колонка грида
+        :type column: ExtGridColumn или None
+        :param colspan: Количество колонок, кот-е находятся под данной колонкой
+        :type colspan: int
+        :param level: Уровень ячейки (0 - самый верхний, 1-ниже, и т.д)
+        :type level: int
 
-        upd:26.10.2010 kirov
-        колонка может быть не указана, т.е. None,
-        в этом случае на указанном уровне будет "дырка"
-        '''
+        .. note ::
+            upd:26.10.2010 kirov
+            колонка может быть не указана, т.е. None,
+            в этом случае на указанном уровне будет "дырка"
+        """
         class BlankBandColumn():
             colspan = 0
 
@@ -312,9 +186,9 @@ class ExtGrid(BaseExtPanel):
         self.show_banded_columns = True
 
     def clear_banded_columns(self):
-        '''
+        """
         Удаляет все объединенные колонки из грида
-        '''
+        """
         self.banded_columns.clear()
         self.show_banded_columns = False
 
@@ -326,21 +200,17 @@ class ExtGrid(BaseExtPanel):
 
     store = property(get_store, set_store)
 
-    def make_read_only(
-            self, access_off=True, exclude_list=[], *args, **kwargs):
-        # Описание в базовом классе ExtUiComponent.
-        # Обрабатываем исключения.
-        access_off = self.pre_make_read_only(
-            access_off, exclude_list, *args, **kwargs)
-        # Выключаем\включаем компоненты.
-        super(ExtGrid, self).make_read_only(
+    def _make_read_only(
+            self, access_off=True, exclude_list=(), *args, **kwargs):
+        super(ExtGrid, self)._make_read_only(
             access_off, exclude_list, *args, **kwargs)
         self.read_only = access_off
         if self.columns:
             for column in self.columns:
                 column.make_read_only(
                     self.read_only, exclude_list, *args, **kwargs)
-        #убираем редактирование записи по даблклику
+
+        # убираем редактирование записи по даблклику
         self.handler_dblclick = 'Ext.emptyFn'
 
         # контекстное меню.
@@ -396,9 +266,7 @@ class ExtGrid(BaseExtPanel):
         self.__cm = value
         self.__cm.grid = self
 
-    #//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\//\\
-    # Врапперы над событиями listeners[...]
-    #------------------------------------------------------------------------
+
     @property
     def handler_click(self):
         return self._listeners.get('click')
@@ -451,7 +319,6 @@ class ExtGrid(BaseExtPanel):
             ('autoExpandColumn', self.auto_expand_column),
             ('editor', self.editor),
             ('view', self.t_render_view, self.view),
-            ('colModel', self.t_render_col_model, self.col_model),
             ('store', self.t_render_store, self.get_store()),
             ('viewConfig', self._view_config),
             ('columnLines', self.column_lines, self.column_lines),
@@ -516,25 +383,10 @@ class ExtGrid(BaseExtPanel):
         return 'createGridPanel({%s}, {%s})' % (config, params)
 
 
-#==============================================================================
-# Оси к пивот гриду
-#==============================================================================
-class ExtPivotGridAxis(ExtUIComponent):
-    DEFAULT_HEADER_WIDTH = 100
-
-    def __init__(self, *args, **kwargs):
-        super(ExtPivotGridAxis, self).__init__(*args, **kwargs)
-        self.width = ExtPivotGridAxis.DEFAULT_HEADER_WIDTH
-        self.data_index = None
-        self.orientation = 'horizontal'
-        self.header = None
-        self.default_header_width = 80
-        self.init_component(*args, **kwargs)
-
-
-#==============================================================================
 class BaseExtGridColumn(ExtUIComponent):
-
+    """
+    Базовая модель колонки грида
+    """
     # Умолчательная ширина колонок
     GRID_COLUMN_DEFAULT_WIDTH = 100
 
@@ -610,12 +462,8 @@ class BaseExtGridColumn(ExtUIComponent):
     def render_editor(self):
         return self.editor.render()
 
-    def make_read_only(
-            self, access_off=True, exclude_list=[], *args, **kwargs):
-        # Описание в базовом классе ExtUiComponent.
-        # Обрабатываем исключения.
-        access_off = self.pre_make_read_only(
-            access_off, exclude_list, *args, **kwargs)
+    def _make_read_only(
+            self, access_off=True, exclude_list=(), *args, **kwargs):
         self.read_only = access_off
         if self.editor and isinstance(self.editor, ExtUIComponent):
             self.editor.make_read_only(
@@ -661,9 +509,9 @@ class BaseExtGridColumn(ExtUIComponent):
         self._column_renderer.append(value)
 
     def render_column_renderer(self):
-        '''
+        """
         Кастомный рендеринг функций-рендерера колонок
-        '''
+        """
         if self._column_renderer:
             self._column_renderer.reverse()
             val = self._get_renderer_func(self._column_renderer)
@@ -674,9 +522,9 @@ class BaseExtGridColumn(ExtUIComponent):
         return None
 
     def _get_renderer_func(self, list_renderers):
-        '''
+        """
         Рекурсивная функция, оборачивающая друг в друга рендереры колонок
-        '''
+        """
         if list_renderers:
             return '%s(%s, metaData, record, rowIndex, colIndex, store)' % (
                 list_renderers[0],
@@ -687,6 +535,9 @@ class BaseExtGridColumn(ExtUIComponent):
 
 
 class ExtGridColumn(BaseExtGridColumn):
+    """
+    Модель колонки грида
+    """
     def __init__(self, *args, **kwargs):
         super(ExtGridColumn, self).__init__(*args, **kwargs)
         self.init_component(*args, **kwargs)
@@ -703,6 +554,9 @@ class ExtGridColumn(BaseExtGridColumn):
 
 
 class ExtGridBooleanColumn(BaseExtGridColumn):
+    """
+    Модель булевой колонки грида
+    """
     def __init__(self, *args, **kwargs):
         super(ExtGridBooleanColumn, self).__init__(*args, **kwargs)
         self.template = 'ext-grids/ext-bool-column.js'
@@ -713,6 +567,9 @@ class ExtGridBooleanColumn(BaseExtGridColumn):
 
 
 class ExtGridCheckColumn(BaseExtGridColumn):
+    """
+    Модель колонки грида, содержащей чекбоксы
+    """
     def __init__(self, *args, **kwargs):
         super(ExtGridCheckColumn, self).__init__(*args, **kwargs)
         self.template = 'ext-grids/ext-check-column.js'
@@ -720,6 +577,9 @@ class ExtGridCheckColumn(BaseExtGridColumn):
 
 
 class ExtGridNumberColumn(BaseExtGridColumn):
+    """
+    Модель колонки грида, содержащей числа
+    """
     def __init__(self, *args, **kwargs):
         super(ExtGridNumberColumn, self).__init__(*args, **kwargs)
         self.template = 'ext-grids/ext-number-column.js'
@@ -728,18 +588,24 @@ class ExtGridNumberColumn(BaseExtGridColumn):
 
 
 class ExtGridDateColumn(BaseExtGridColumn):
+    """
+    Модель колонки грида с форматом даты
+    """
     def __init__(self, *args, **kwargs):
         super(ExtGridDateColumn, self).__init__(*args, **kwargs)
         self.template = 'ext-grids/ext-date-column.js'
         try:
             self.format = settings.DATE_FORMAT.replace('%', '')
-        except:
+        except AttributeError:
             self.format = 'd.m.Y'
 
         self.init_component(*args, **kwargs)
 
 
 class BaseExtGridSelModel(BaseExtComponent):
+    """
+    Базовая модель для грида с выбором
+    """
     def __init__(self, *args, **kwargs):
         super(BaseExtGridSelModel, self).__init__(*args, **kwargs)
 
@@ -804,7 +670,7 @@ class ExtGridDefaultColumnModel(BaseExtComponent):
 
 class ExtGridLockingColumnModel(BaseExtComponent):
     """
-    Модель колонок для грида блокрирования
+    Модель колонок для грида блокирования
     """
     # TODO: Этот класс, т.к. ссылка на грид порождает цикличную связь
     def __init__(self, *args, **kwargs):
@@ -817,85 +683,6 @@ class ExtGridLockingColumnModel(BaseExtComponent):
             self.grid.t_render_columns())
 
 
-class ExtGridLockingHeaderGroupColumnModel(BaseExtComponent):
-    # TODO: Этот класс, т.к. ссылка на грид порождает цикличную связь
-    def __init__(self, *args, **kwargs):
-        super(ExtGridLockingHeaderGroupColumnModel, self).__init__(
-            *args, **kwargs)
-        self.grid = None
-        self.init_component(*args, **kwargs)
-
-    def render(self):
-        return 'new Ext.ux.grid.LockingGroupColumnModel({columns:%s})' % (
-            self.grid.t_render_columns())
-
-
-class ExtAdvancedTreeGrid(ExtGrid):
-    """
-    Расширенное дерево на базе Ext.ux.maximgb.TreeGrid
-    """
-    def __init__(self, *args, **kwargs):
-        super(ExtAdvancedTreeGrid, self).__init__(*args, **kwargs)
-        self.template = 'ext-grids/ext-advanced-treegrid.js'
-        self.url = None
-        self.master_column_id = None
-
-        # Свойства для внутреннего store:
-        self.store_root = 'rows'
-
-        # Свойства для внутеннего bottom bara:
-        self.use_bbar = False
-
-        # Количество записей
-        self.bbar_page_size = 10
-
-        self.init_component(*args, **kwargs)
-
-    def t_render_columns_to_record(self):
-        return '[%s]' % ','.join([
-            '{name:"%s"}' % col.data_index
-            for col in self.columns
-        ])
-
-    def add_column(self, **kwargs):
-        # FIXME: Хак, с сгенерированным client_id
-        # компонент отказывается работать
-        if kwargs.get('data_index'):
-            kwargs['client_id'] = kwargs.get('data_index')
-        super(ExtAdvancedTreeGrid, self).add_column(**kwargs)
-
-    def render_base_config(self):
-        super(ExtAdvancedTreeGrid, self).render_base_config()
-        self._put_config_value('master_column_id', self.master_column_id)
-
-    def render_params(self):
-        super(ExtAdvancedTreeGrid, self).render_params()
-        self._put_params_value(
-            'storeParams',
-            {
-                'url': self.url,
-                'root': self.store_root
-            }
-        )
-        self._put_params_value(
-            'columnsToRecord', self.t_render_columns_to_record)
-
-        if self.use_bbar:
-            self._put_params_value('bbar', {'pageSize': self.bbar_page_size})
-
-    def t_render_base_config(self):
-        return self._get_config_str()
-
-    def render(self):
-        self.render_base_config()
-        self.render_params()
-
-        base_config = self._get_config_str()
-        params = self._get_params_str()
-        return 'createAdvancedTreeGrid({%s},{%s})' % (
-            base_config, params)
-
-
 class ExtGridGroupingView(BaseExtComponent):
     """
     Компонент используемый для группировки
@@ -929,7 +716,7 @@ class ExtGridGroupingView(BaseExtComponent):
         except UnicodeDecodeError as msg:
             raise Exception(msg)
         params = self._get_params_str()
-        return 'new Ext.grid.GroupingView({%s})' % (params)
+        return 'new Ext.grid.GroupingView({%s})' % params
 
 
 class ExtGridLockingView(BaseExtComponent):
@@ -945,21 +732,6 @@ class ExtGridLockingView(BaseExtComponent):
         return result
 
 
-class ExtGridLockingHeaderGroupView(BaseExtComponent):
-    """
-    Компонент используемый для блокирования колонок и их группировки
-    """
-    def __init__(self, *args, **kwargs):
-        super(ExtGridLockingHeaderGroupView, self).__init__(*args, **kwargs)
-        self.grid = None
-        self.init_component(*args, **kwargs)
-
-    def render(self):
-        result = 'new Ext.ux.grid.LockingHeaderGroupGridView({rows:%s})' % (
-            self.grid.t_render_banded_columns())
-        return result
-
-
 class ExtLiveGridCheckBoxSelModel(ExtGridCheckBoxSelModel):
     """
     Модель выбора для live-грида с возможностью отметки чек-боксом
@@ -990,3 +762,63 @@ class ExtLiveGridRowSelModel(ExtGridRowSelModel):
         single_sel = 'singleSelect: true' if self.single_select else ''
         return 'new Ext.ux.grid.livegrid.RowSelectionModel({ %s })' % (
             single_sel)
+
+
+class ExtGridLockingHeaderGroupPlugin(BaseExtComponent):
+    """
+    Плагин для группировки и одновременного закрепления колонок
+    """
+
+    def __init__(self, config):
+        """
+        :param dict config: Конфигурация плагина, описание выше
+        """
+        super(ExtGridLockingHeaderGroupPlugin, self).__init__()
+        self._ext_name = 'Ext.ux.grid.LockingGridColumnWithHeaderGroup'
+        self.config = config
+
+    def render(self):
+        return 'new %s(%s)' % (self._ext_name, json.dumps(self.config))
+
+    @classmethod
+    def configure_grid(cls, grid, locked_count=1, config=None):
+        """
+        Конфигурирует grid для работы с собой, как плагином
+        :param grid: Конфигурируемый grid
+        :type grid: ExtGrid
+        :param locked_count: Кол-во фиксируемых колонок
+        :type locked_count: int
+        :param config: дополнительная конфигурация
+        :type config: dict
+        """
+        # адаптация колонок
+        rows = [
+            [
+                {'header': c.header, 'colspan': c.colspan, 'align': c.align}
+                for c in cs
+            ]
+            for cs in grid.banded_columns.values()
+        ]
+        grid.banded_columns = {}
+
+        # настройка плагина
+        plugin_config = {
+            'columnModelCfg': {
+                'rows': rows,
+                'lockedCount': locked_count
+            },
+            'viewCfg': {
+                'hideGroupedColumn': True
+            }
+        }
+        if config is not None:
+            plugin_config.update(config)
+        grid.plugins.append(cls(plugin_config))
+
+        # настройка store
+        from m3_ext.ui.misc.store import ExtGroupingStore, ExtJsonReader
+        store = ExtGroupingStore(
+            url=grid.store.url, auto_load=grid.store.auto_load)
+        store.reader = ExtJsonReader(total_property='total', root='rows')
+        store.reader.set_fields(*grid.columns)
+        grid.set_store(store)
